ScornedMacros_Setup = {
};

function ScornedMacros_AddEventHandler(event, func)
	ScornedMacrosFrame:RegisterEvent(event);
	if ( not ScornedMacros_Setup.eventHandlers ) then
		ScornedMacros_Setup.eventHandlers = {};
	end
	local eventHandlers = ScornedMacros_Setup.eventHandlers[event];
	if ( not eventHandlers ) then
		eventHandlers = {};
		ScornedMacros_Setup.eventHandlers[event] = eventHandlers;
	end
	eventHandlers[func] = true;
end

function ScornedMacros_RemoveEventHandler(event, func)
	if ( not ScornedMacros_Setup.eventHandlers ) then
		return;
	end
	local eventHandlers = ScornedMacros_Setup.eventHandlers[event];
	if ( not eventHandlers ) then
		return;
	end
	eventHandlers[func] = nil;
end

function ScornedMacros_OnEvent()
	if ( not ScornedMacros_Setup.eventHandlers ) then
		return;
	end
	local eventHandlers = ScornedMacros_Setup.eventHandlers[event];
	if ( not eventHandlers ) then
		return;
	end
	for k,v in pairs(eventHandlers) do
		if ( type(k) == "function" ) and ( v ) then
			k();
		end
	end
end

function ScornedMacros_InitiateFrame(frame)
	frame:SetScript("OnEvent", ScornedMacros_OnEvent);
	ScornedMacros_AddEventHandler("SPELLS_CHANGED", ScornedMacros_OnSpellsChanged);
end

function ScornedMacros_OnSpellsChanged()
	ScornedMacros_Setup.spellIds = {};
end

function ScornedMacros_GetSpellId(spellName, spellBook)
	if ( not ScornedMacros_Setup.spellIds ) then
		ScornedMacros_Setup.spellIds = {};
	end
	if ( spellBook == BOOKTYPE_SPELL ) then
		local id = ScornedMacros_Setup.spellIds[spellName];
		if ( not id ) then
			id = ScornedMacros_RetrieveSpellId(spellName, spellBook);
			if ( id == nil ) then
				id = false;
			end
			ScornedMacros_Setup.spellIds[spellName] = id;
		end
		if ( id == false ) then 
			return nil;
		else
			return id;
		end
	end
	return ScornedMacros_RetrieveSpellId(spellName, spellBook);
end

function ScornedMacros_RetrieveSpellId(spellName, spellBook)
	if ( not spellName ) then return nil; end
	if ( not spellBook ) then spellBook = BOOKTYPE_SPELL; end
	local name, rank;
	local foundId = nil;
	for i=1, MAX_SPELLS do
		name, rank = GetSpellName(spellName, spellBook);
		if ( name ) and ( name == spellName ) then
			foundId = i;
		elseif ( foundId ) then
			break;
		end
	end
	return foundId;
end


function ScornedMacros_HasUnitBuff(unit, icon, buffName)
	if ( not unit ) or ( not icon ) then
		return false;
	end
	local hasBuff = false;
	if ( icon ) then
		local buff = nil;
		for i=1,32 do
			buff = UnitBuff(unit, i);
			if ( not buff ) then
				break;
			elseif ( buff == icon ) then
				-- TODO: add check of buffName here.
				hasBuff = true;
				break;
			end
		end
	end
	return hasBuff;
end

function ScornedMacros_UseSpellOnUnit(spellName, unit)
	if ( not unit ) or ( not spellName ) then
		return false;
	end
	local onSelf = nil;
	if ( unit == "player" ) then
		onSelf = 1;
	end
	CastSpellByName(spellName, onSelf);
	if ( not onSelf ) and ( unit ~= "target" ) then SpellTargetUnit(unit); end
	return true;
end

function ScornedMacros_UseSpellIfNotHasBuff(unit, spellName, icon, buffName)
	if ( not unit ) or ( not spellName ) then
		return false;
	end
	if ( not buffName ) then buffName = spellName; end
	local hasBuff = ScornedMacros_HasUnitBuff(unit, icon, buffName);
	if ( not hasBuff ) then
		return ScornedMacros_UseSpellOnUnit(spellName, unit);
	end
	return false;
end


local f=CreateFrame("Frame","ScornedMacrosFrame");
ScornedMacros_InitiateFrame(f);
